package com.unissoft.mqtt;

import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * @author qingsong.han
 * @description: ScheduledExecutorService
 * @create 2022-10-25 15:13
 */
@Component
@Order(10)
public class MqttSenderSchedule implements ApplicationRunner {
    @Resource
    private MqttConfig.MqttMessageSender messageSender;

    private void sendToMqtt() {
        // 创建一个ScheduledThreadPoolExecutor线程池，核心线程数为5
        ScheduledExecutorService scheduledExecutorService = new ScheduledThreadPoolExecutor(4);
        // 创建Runnable打印当前线程和当前时间
//        Runnable r = () -> System.out.println("当前线程：" + Thread.currentThread().getName() + " 当前时间" + LocalDateTime.now());
        Runnable r = () -> messageSender.sendToMqtt(
                // topic
                "/PUSH/8951DQ2207200018/DPUT/upload",
                // qos
                1,
                // payload
                "{\"data\":[{\"device\":\"8951D10\",\"tempValue\":\"20\",\"wetValue\":\"64\",\"dewValue\":\"13\"},{\"device\":\"8951D11\",\"tempValue\":\"19\",\"wetValue\":\"66\",\"dewValue\":\"13\"},{\"device\":\"8951D12\",\"tempValue\":\"19\",\"wetValue\":\"65\",\"dewValue\":\"13\"},{\"device\":\"8951D13\",\"tempValue\":\"19\",\"wetValue\":\"67\",\"dewValue\":\"13\"},{\"device\":\"8951D14\",\"tempValue\":\"20\",\"wetValue\":\"65\",\"dewValue\":\"13\"},{\"device\":\"8951D15\",\"tempValue\":\"21\",\"wetValue\":\"61\",\"dewValue\":\"13\"}],\"time\":\"2018-03-18 19:00:43\"}"
        );
        /*
         * schedule：只执行一次调度
         * scheduleAtFixedRate：一开始就计算间隔时间，如果任务超过间隔时间，那么就直接开始下一个任务
         * scheduleWithFixedDelay：任务无论执行多久，都要等待上一轮任务完成之后再间隔指定时间，然后才开始下一个任务
         */
        // 在指定10秒延迟后执行r,之后每10秒执行一次
        scheduledExecutorService.scheduleAtFixedRate(r, 10, 10, TimeUnit.SECONDS);

        // --------------------------------------------------------------------------------------
        Runnable r1 = () -> messageSender.sendToMqtt(
                // topic
                "/PUSH/8951DQ2207200019/DPUT/upload",
                // qos
                1,
                // payload
                "{\"data\":[{\"device\":\"8951D10\",\"tempValue\":\"25\",\"wetValue\":\"62\",\"dewValue\":\"17\"},{\"device\":\"8951D11\",\"tempValue\":\"23\",\"wetValue\":\"63\",\"dewValue\":\"15\"},{\"device\":\"8951D12\",\"tempValue\":\"26\",\"wetValue\":\"55\",\"dewValue\":\"16\"},{\"device\":\"8951D13\",\"tempValue\":\"22\",\"wetValue\":\"66\",\"dewValue\":\"18\"},{\"device\":\"8951D14\",\"tempValue\":\"23\",\"wetValue\":\"57\",\"dewValue\":\"14\"},{\"device\":\"8951D15\",\"tempValue\":\"28\",\"wetValue\":\"67\",\"dewValue\":\"20\"}],\"time\":\"2019-04-28 20:01:00\"}"
        );
        scheduledExecutorService.scheduleAtFixedRate(r1, 11, 10, TimeUnit.SECONDS);
        // -------------------------------------------------------------------------------------------

        Runnable r2 = () -> messageSender.sendToMqtt(
                // topic
                "/PUSH/8951DQ2207200020/DPUT/upload",
                // qos
                1,
                // payload
                "{\"data\":[{\"device\":\"8951D16\",\"tempValue\":\"27\",\"wetValue\":\"64\",\"dewValue\":\"21\"},{\"device\":\"8951D17\",\"tempValue\":\"30\",\"wetValue\":\"77\",\"dewValue\":\"26\"},{\"device\":\"8951D18\",\"tempValue\":\"26\",\"wetValue\":\"68\",\"dewValue\":\"19\"},{\"device\":\"8951D19\",\"tempValue\":\"26\",\"wetValue\":\"59\",\"dewValue\":\"18\"},{\"device\":\"8951D20\",\"tempValue\":\"22\",\"wetValue\":\"56\",\"dewValue\":\"16\"},{\"device\":\"8951D21\",\"tempValue\":\"23\",\"wetValue\":\"17\",\"dewValue\":\"13\"}],\"time\":\"2019-04-28 20:01:10\"}"
        );
        // 在指定10秒延迟后执行r,之后每10秒执行一次
        scheduledExecutorService.scheduleAtFixedRate(r2, 12, 10, TimeUnit.SECONDS);
        // -------------------------------------------------------------------------------------------
    }

    @Override
    public void run(ApplicationArguments args) throws Exception {
        this.sendToMqtt();
    }

}
